package acoes;

import java.awt.Dimension;
import java.awt.Rectangle;
import java.awt.Robot;
import java.awt.Toolkit;
import java.awt.event.InputEvent;
import java.awt.image.BufferedImage;

import javafx.beans.property.ObjectProperty;
import javafx.beans.property.SimpleObjectProperty;
import main.BuscadorDaCheckboxDaLinhaSelecionada;
import main.MainFX;

public class ClicarNaCheckbox extends Acao {

	private BuscadorDaCheckboxDaLinhaSelecionada buscadorDaCheckboxDaLinhaSelecionada;
	
	public static enum Resultado {CLICOU, NAO_CLICOU, NAO_ATIVADO}
	public final ObjectProperty<Resultado> clicouNaCheckbox = new SimpleObjectProperty<>(Resultado.NAO_ATIVADO);

	public ClicarNaCheckbox(BuscadorDaCheckboxDaLinhaSelecionada buscadorDaCheckboxDaLinhaSelecionada) {
		this.buscadorDaCheckboxDaLinhaSelecionada = buscadorDaCheckboxDaLinhaSelecionada;
		
		clicouNaCheckbox.addListener((obs, old, nv) -> {
			if (nv != Resultado.NAO_ATIVADO)
				restaurarEstadoNeutroApos(1000);
		});
	}
	
	private void restaurarEstadoNeutroApos(long millis) {
		new Thread(() -> {
			try {
				Thread.sleep(millis);
			} catch (InterruptedException e1) {
				e1.printStackTrace();
			}
			clicouNaCheckbox.set(Resultado.NAO_ATIVADO);
		}).start();
	}

	/**
	 * Retorna true apenas se encontrou e clicou na Checkbox, false caso contrrio.
	 * @param robo 
	 * @return 
	 */
	@Override
	public boolean executar(Robot robo) {
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		BufferedImage screenCapture = robo.createScreenCapture(new Rectangle(0, 0, screenSize.width, screenSize.height));
		Rectangle boundsDaCheckboxEncontrada = buscadorDaCheckboxDaLinhaSelecionada.buscarCheckboxDaLinhaSelecionada(
				screenCapture);
		System.out.println(boundsDaCheckboxEncontrada);
		if (boundsDaCheckboxEncontrada != null) {
			int centroDaCheckBox_X = boundsDaCheckboxEncontrada.x + (boundsDaCheckboxEncontrada.width / 2);
			int centroDaCheckBox_Y = boundsDaCheckboxEncontrada.y + (boundsDaCheckboxEncontrada.height / 2);
			clicar(centroDaCheckBox_X, centroDaCheckBox_Y, robo);
			System.out.println("Clicou na Checkbox - x: " + centroDaCheckBox_X + ", y: " + centroDaCheckBox_Y);
			clicouNaCheckbox.set(Resultado.CLICOU);
			return true;
		}
		clicouNaCheckbox.set(Resultado.NAO_CLICOU);
		atualizarGUI();
		return false;
	}

	private void atualizarGUI() {
		MainFX.mostrarTelaDaCaptura();
	}

	/** Clica na posio x e y passadas. 
	 * @param robo */
	private void clicar(int x, int y, Robot robo) {
		robo.mouseMove(x, y);
		robo.mousePress(InputEvent.BUTTON1_DOWN_MASK);
		robo.mouseRelease(InputEvent.BUTTON1_DOWN_MASK);
	}

}
