package acoes;

import java.awt.Rectangle;
import java.awt.Robot;
import java.io.File;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.List;

import javax.imageio.ImageIO;

import acoes.Teclar.TipoDoEvento;
import javafx.beans.property.BooleanProperty;
import javafx.beans.property.ReadOnlyBooleanProperty;
import javafx.beans.property.SimpleBooleanProperty;
import main.Carregador;

public class SequenciaDeAcoes extends Acao implements Cancelavel {
	
	private List<Acao> sequencia = new ArrayList<>();
	
	private volatile boolean isCancelar = false;
	private volatile Acao acaoAtual;
	
	private final BooleanProperty emExecucao = new SimpleBooleanProperty(false);
	public final ReadOnlyBooleanProperty ro_emExecucao = emExecucao;

	@Override
	public boolean executar(Robot robo) {
		if (emExecucao.get()) throw new IllegalStateException("J est em Execuo!");
		emExecucao.set(true);
		isCancelar = false;
		Thread thread = new Thread(() -> {
			for (Acao acao : sequencia) {
				gui.clear();
				gui.addAll(acao.getGUI());
				acaoAtual = acao;
				if (!acao.executar(robo)) break;
				if (isCancelar) break;
			}
			gui.clear();
			acaoAtual = null;
			emExecucao.set(false);
		});
		thread.start();
		return false;
	}
	
	@Override
	public void cancelar() {
		isCancelar = true;
		if (acaoAtual instanceof Cancelavel) {
			((Cancelavel) acaoAtual).cancelar();
		}
	}

	
	public static SequenciaDeAcoes carregarListaDeAcoes(ClicarNaCheckbox clicarNaCheckbox) throws IOException {
		System.out.println("---------------------------------------------------------------------------------");
		System.out.println("Sequncia de Aes:");
		SequenciaDeAcoes sequenciaDeAcoes = new SequenciaDeAcoes();
		Files.lines(Paths.get("src/sequencia_de_acoes.txt"), StandardCharsets.ISO_8859_1).forEach(linha -> {
			if (!linha.isEmpty() && !linha.trim().startsWith("//")) {
				System.out.println(linha);
				
				if (linha.contains("CLICAR_NA_CHECKBOX")) {
					sequenciaDeAcoes.sequencia.add(clicarNaCheckbox);
				} else if (linha.contains("DIGITAR_ENTRADA_ATUAL")) {
					sequenciaDeAcoes.sequencia.add(new DigitarEntradaAtual());
				} else if (linha.contains("PROXIMA_ENTRADA")) {
					sequenciaDeAcoes.sequencia.add(new ProximaEntrada());
				} else {
				
					String[] split = linha.trim().split(":");
					String key = split[0].trim();
					String argumentosNumaString = split[1].trim();
					argumentosNumaString = removerAspas(argumentosNumaString);
					System.out.println("'" + key + "':'" + argumentosNumaString + "'");
					
					if (argumentosNumaString.contains(",")) {
						String[] argumentos = argumentosNumaString.split(",");
						
						if (key.contains("CLICAR")) {
							sequenciaDeAcoes.sequencia.add(new Clicar(toInt(argumentos[0]), toInt(argumentos[1])));
						} else if (key.contains("TECLAR")) {
							sequenciaDeAcoes.sequencia.add(new Teclar(toInt(argumentos[0]), TipoDoEvento.valueOf(removerAspas(argumentos[1].trim()).trim())));
						} else if (key.contains("AGUARDAR")) {
							try {
								sequenciaDeAcoes.sequencia.add(new AguardarImagem(ImageIO.read(new File(removerAspas(argumentos[0].trim()))), 
										new Rectangle(toInt(argumentos[1]), toInt(argumentos[2]), toInt(argumentos[3]), toInt(argumentos[4])))
										);
							} catch (IOException e) {
								Carregador.saidaComErro(e);
							}
						}
					} else if (key.contains("AGUARDAR")) {
						sequenciaDeAcoes.sequencia.add(new Aguardar(Long.parseLong(removerAspas(argumentosNumaString.trim()).trim())));
					}
				}
			}
		});
		System.out.println("---------------------------------------------------------------------------------");
		return sequenciaDeAcoes;
	}

	private static int toInt(String argumento) {
		return Integer.parseInt(removerAspas(argumento.trim()).trim());
	}

	private static String removerAspas(String values) {
		if (values.startsWith("\"") || values.startsWith("\'") || values.startsWith("")) {
			values = values.substring(1);
		}
		if (values.endsWith("\"") || values.endsWith("\'") || values.endsWith("")) {
			values = values.substring(0, values.length() - 1);
		}
		return values;
	}
}
