package main;

import java.awt.Dimension;
import java.awt.Rectangle;
import java.awt.image.BufferedImage;

public class BuscadorDeImagemDentroDeImagem {

	/**
	 * Busca a 'imagemFilho' dentro da 'imagemPai' retornando um {@link Rectangle}
	 * com a informao (x, y, width e height) de onde ela foi encontrada na
	 * 'imagemPai'; ou retorna null caso no a encontre.
	 */
	public static Rectangle buscarFilho(BufferedImage imagemPai, BufferedImage imagemFilho) {
		for (int x = 0; x < imagemPai.getWidth(); x++) {
			for (int y = 0; y < imagemPai.getHeight(); y++) {
				if (contemFilho(imagemPai, imagemFilho, x, y)) {
					return new Rectangle(x, y, imagemFilho.getWidth(), imagemFilho.getHeight());
				}
			}
		}
		return null;
	}

	/**
	 * Retorna true apenas se a 'imagemFilho' for encontrada na 'imagemPai' na
	 * posio 'offsetX' e 'offsetY' passadas; retorna false caso contrrio.
	 */
	public static boolean contemFilho(BufferedImage imagemPai, BufferedImage imagemFilho, int offsetX, int offsetY) {
		if (!filhoCabeDentroDoPai(getDimensoes(imagemPai), getDimensoes(imagemFilho), offsetX, offsetY)) {
			return false; // no contm o filho se ele no cabe dentro do pai
		}
		for (int x = 0; x < imagemFilho.getWidth(); x++) {
			for (int y = 0; y < imagemFilho.getHeight(); y++) {
//				System.out.println(
//						"x: "+x+"  --  y: "+y+"  --  offsetX: "+offsetX+"  --  offsetY: "+offsetY+
//						"  --  offsetX + x: "+(offsetX + x)+"  --  offsetY + y: "+(offsetY + y));
				if (imagemPai.getRGB(offsetX + x, offsetY + y) 
						!= imagemFilho.getRGB(x, y)) {
					return false;
				}
			}
		}
		return true;
	}

	private static Dimension getDimensoes(BufferedImage imagem) {
		return new Dimension(imagem.getWidth(), imagem.getHeight());
	}

	static boolean filhoCabeDentroDoPai(Dimension pai, Dimension filho, int offsetX, int offsetY) {
		if ((offsetX + filho.getWidth()) > pai.getWidth()) {
			return false;
		}
		if ((offsetY + filho.getHeight()) > pai.getHeight()) {
			return false;
		}
		return true;
	}

}
