package main;

import java.awt.Color;
import java.awt.HeadlessException;
import java.awt.Rectangle;
import java.awt.Robot;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.HashMap;
import java.util.Map;

import javax.imageio.ImageIO;

import acoes.ClicarNaCheckbox;
import acoes.SequenciaDeAcoes;
import javafx.geometry.Pos;
import javafx.scene.control.Alert;
import javafx.scene.control.Alert.AlertType;
import javafx.scene.control.Label;
import javafx.scene.control.TextArea;
import javafx.scene.layout.GridPane;
import javafx.scene.layout.Priority;

public class Carregador {

	static BufferedImage checkBoxImage;
	static Robot robo;

	static BuscadorDaCheckboxDaLinhaSelecionada buscadorDaCheckboxDaLinhaSelecionada;
	static ClicarNaCheckbox clicador;

	static int larguraDaJanelaDeEntradas;
	static int alturaDaJanelaDeEntradas;
	static int posXdaJanelaDeEntradas;
	static int posYdaJanelaDeEntradas;
	
	static SequenciaDeAcoes sequenciaDeAcoes;
	
	static int posYdaCaixaDeBotoes;
	static int posXdaCaixaDeBotoes;
	static int larguraDaJanelaDeCapturas;
	static int alturaDaJanelaDeCapturas;
	static Pos alinhamentoDosBotoes;
	
	public static boolean mostrarImagemBuscada;
	public static int margemHorizontalDaImagemBuscada;
	public static int margemVerticalDaImagemBuscada;
	public static boolean imagemBuscadaAncoradaNoTopo;
	public static boolean imagemBuscadaAncoradaNaDireita;

	public static void iniciarTudo() {
		try {
			iniciarConfiguracoes();
			clicador = new ClicarNaCheckbox(buscadorDaCheckboxDaLinhaSelecionada);
			sequenciaDeAcoes = SequenciaDeAcoes.carregarListaDeAcoes(clicador);
		} catch (Exception e1) {
			e1.printStackTrace();
			saidaComErro(e1);
		}
	}

	private static void iniciarConfiguracoes() throws Exception {
		robo = new Robot();
		robo.setAutoWaitForIdle(true);
		carregarConfiguracoesDoArquivo();
	}

	private static void carregarConfiguracoesDoArquivo() throws IOException {
		carregarConfiguracaoesAPartirDoMap(carregarMapDeConfiguracoes());
	}

	private static void carregarConfiguracaoesAPartirDoMap(Map<String, String> configuracoes) throws IOException {
		Conversor c = new Conversor(configuracoes);

		checkBoxImage = ImageIO.read(new File(configuracoes.get("IMAGEM_DE_UMA_CHECKBOX")));

		BuscadorDeLinhaSelecionada buscadorDeLinhaSelecionada = new BuscadorDeLinhaSelecionada(
				new Color(
						c.integerConfig("COR_DA_SELECAO_RED"), 
						c.integerConfig("COR_DA_SELECAO_GREEN"), 
						c.integerConfig("COR_DA_SELECAO_BLUE")), 
				c.integerConfig("MARGEM_SUPERIOR_ADICIONADA_A_AREA_DA_CHECKBOX_BUSCADA"), 
				c.integerConfig("MARGEM_INFERIOR_ADICIONADA_A_AREA_DA_CHECKBOX_BUSCADA"));

		Rectangle areaDaSelecao = new Rectangle(
				c.integerConfig("POSICAO_X_AREA_DA_SELECAO"), 
				c.integerConfig("POSICAO_Y_AREA_DA_SELECAO"), 
				c.integerConfig("LARGURA_DA_AREA_DA_SELECAO"), 
				c.integerConfig("ALTURA_DA_AREA_DA_SELECAO"));

		buscadorDaCheckboxDaLinhaSelecionada = new BuscadorDaCheckboxDaLinhaSelecionada(
				checkBoxImage, buscadorDeLinhaSelecionada, areaDaSelecao, 
				c.integerConfig("BORDA_ESQUERDA_DA_AREA_DE_CHECKBOX"), 
				c.integerConfig("BORDA_DIREITA_DA_AREA_DE_CHECKBOX"));
		
		larguraDaJanelaDeCapturas = c.integerConfig("LARGURA_DA_JANELA_DE_CAPTURAS");
		alturaDaJanelaDeCapturas = c.integerConfig("ALTURA_DA_JANELA_DE_CAPTURAS");

		larguraDaJanelaDeEntradas = c.integerConfig("LARGURA_DA_JANELA_DE_ENTRADAS");
		alturaDaJanelaDeEntradas = c.integerConfig("ALTURA_DA_JANELA_DE_ENTRADAS");
		posXdaJanelaDeEntradas = c.integerConfig("POSICAO_X_DA_JANELA_DE_ENTRADAS");
		posYdaJanelaDeEntradas = c.integerConfig("POSICAO_Y_DA_JANELA_DE_ENTRADAS");
		
		posXdaCaixaDeBotoes = c.integerConfig("POSICAO_X_DA_CAIXA_DE_BOTOES");
		posYdaCaixaDeBotoes = c.integerConfig("POSICAO_Y_DA_CAIXA_DE_BOTOES");
		
		if (configuracoes.containsKey("ROBOT_AUTO_DELAY"))
			robo.setAutoDelay(c.integerConfig("ROBOT_AUTO_DELAY"));
		
		alinhamentoDosBotoes = Pos.CENTER_LEFT;
		String alinhamento = configuracoes.get("ALINHAMENTO_DOS_BOTOES");
		if (alinhamento != null) {
			alinhamento = alinhamento.toUpperCase();
			if (alinhamento.contains("DI") || alinhamento.contains("RI"))
				alinhamentoDosBotoes = Pos.CENTER_RIGHT;
			if (alinhamento.contains("CE") || alinhamento.contains("ME") || alinhamento.contains("MI"))
				alinhamentoDosBotoes = Pos.CENTER;
		}
		
		mostrarImagemBuscada = true;
		String mostrarImgBuscada = configuracoes.get("MOSTRAR_IMAGEM_BUSCADA");
		if (mostrarImgBuscada != null) {
			mostrarImgBuscada = mostrarImgBuscada.toUpperCase();
			if (mostrarImgBuscada.contains("F") || mostrarImgBuscada.contains("N") || mostrarImgBuscada.contains("OC")  || mostrarImgBuscada.contains("H"))
				mostrarImagemBuscada = false;
		}
		
		margemHorizontalDaImagemBuscada = c.integerConfig("MARGEM_HORIZONTAL_DA_IMAGEM_BUSCADA");
		margemVerticalDaImagemBuscada = c.integerConfig("MARGEM_VERTICAL_DA_IMAGEM_BUSCADA");
		
		imagemBuscadaAncoradaNoTopo = true;
		String ancoraVerticalDaImgBuscada = configuracoes.get("ANCORA_VERTICAL_DA_IMAGEM_BUSCADA");
		if (ancoraVerticalDaImgBuscada != null) {
			ancoraVerticalDaImgBuscada = ancoraVerticalDaImgBuscada.toUpperCase();
			if (ancoraVerticalDaImgBuscada.contains("B") || ancoraVerticalDaImgBuscada.contains("CH") || ancoraVerticalDaImgBuscada.contains("PI"))
				imagemBuscadaAncoradaNoTopo = false;
		}
		
		imagemBuscadaAncoradaNaDireita = true;
		String ancoraHorizontalDaImgBuscada = configuracoes.get("ANCORA_HORIZONTAL_DA_IMAGEM_BUSCADA");
		if (ancoraHorizontalDaImgBuscada != null) {
			ancoraHorizontalDaImgBuscada = ancoraHorizontalDaImgBuscada.toUpperCase();
			if (ancoraHorizontalDaImgBuscada.contains("ES") || ancoraHorizontalDaImgBuscada.contains("LE") || ancoraHorizontalDaImgBuscada.contains("EQ"))
				imagemBuscadaAncoradaNaDireita = false;
		}
		
	}

	private static class Conversor {
		Map<String, String> configuracoes;

		Conversor(Map<String, String> configuracoes) {
			this.configuracoes = configuracoes;
		}

		int integerConfig(String key) {
			return Integer.parseInt(configuracoes.get(key));
		}
		
//		long longConfig(String key) {
//			return Long.parseLong(configuracoes.get(key));
//		}
	}

	private static Map<String, String> carregarMapDeConfiguracoes() throws IOException {
		Map<String, String> configuracoes = new HashMap<>();
		Files.lines(Paths.get("src/configuracoes.txt"), StandardCharsets.ISO_8859_1).forEach(linha -> {
			if (!linha.isEmpty() && !linha.startsWith("//")) {
				System.out.println("Carregar Linha: '"+linha+"'");
				String[] split = linha.trim().split(":");
				String key = split[0].trim();
				String value = split[1].trim();
				if (value.startsWith("\"") || value.startsWith("\'") || value.startsWith("")) {
					value = value.substring(1);
				}
				if (value.endsWith("\"") || value.endsWith("\'") || value.endsWith("")) {
					value = value.substring(0, value.length() - 1);
				}
				configuracoes.put(key, value);
			}
		});
		return configuracoes;
	}

	public static void saidaComErro(Exception ex) throws HeadlessException {
		Alert alert = new Alert(AlertType.ERROR);
		alert.setTitle("Exceo Lanada!");
		alert.setHeaderText("Uma Exception ocorreu durante a execuo");
		alert.setContentText(ex.toString());

		// Create expandable Exception.
		StringWriter sw = new StringWriter();
		PrintWriter pw = new PrintWriter(sw);
		ex.printStackTrace(pw);
		String exceptionText = sw.toString();

		Label label = new Label("Stacktrace:");

		TextArea textArea = new TextArea(exceptionText);
		textArea.setEditable(false);
		textArea.setWrapText(true);

		textArea.setMaxWidth(Double.MAX_VALUE);
		textArea.setMaxHeight(Double.MAX_VALUE);
		GridPane.setVgrow(textArea, Priority.ALWAYS);
		GridPane.setHgrow(textArea, Priority.ALWAYS);

		GridPane expContent = new GridPane();
		expContent.setMaxWidth(Double.MAX_VALUE);
		expContent.add(label, 0, 0);
		expContent.add(textArea, 0, 1);

		// Set expandable Exception into the dialog pane.
		alert.getDialogPane().setExpandableContent(expContent);

		alert.showAndWait();
		System.exit(1);
	}

}
