package main;


import java.awt.Rectangle;
import java.awt.Toolkit;

import javafx.application.Application;
import javafx.application.Platform;
import javafx.beans.binding.BooleanBinding;
import javafx.beans.property.BooleanProperty;
import javafx.beans.property.ObjectProperty;
import javafx.beans.property.SimpleBooleanProperty;
import javafx.beans.property.SimpleObjectProperty;
import javafx.beans.property.StringProperty;
import javafx.collections.ListChangeListener;
import javafx.embed.swing.SwingFXUtils;
import javafx.scene.Cursor;
import javafx.scene.Group;
import javafx.scene.Node;
import javafx.scene.Scene;
import javafx.scene.control.Alert;
import javafx.scene.control.Alert.AlertType;
import javafx.scene.control.Button;
import javafx.scene.control.ScrollPane;
import javafx.scene.control.TextArea;
import javafx.scene.control.ToggleButton;
import javafx.scene.image.Image;
import javafx.scene.image.ImageView;
import javafx.scene.layout.AnchorPane;
import javafx.scene.layout.VBox;
import javafx.scene.paint.Color;
import javafx.scene.text.TextAlignment;
import javafx.stage.Stage;
import javafx.stage.StageStyle;
import view.Retangulo;

public class MainFX extends Application {

	private static final Group ROOT = new Group();
	private static Stage TelaDaCaptura;
	private static Group itensCapturados;
	public volatile static String entradaAtual = "";
	private ToggleButton showConfigsButton;
	private ToggleButton showTextArea;
	private VBox caixaDeBotoes;
	private static TextArea textArea;

	@Override
	public void start(Stage primaryStage) {
		Carregador.iniciarTudo();
		
		primaryStage.setMaximized(true);
		primaryStage.setTitle("Clicador");
		primaryStage.initStyle(StageStyle.TRANSPARENT);
		Button processarUmaVezButton = new Button("Processar Sequncia uma vez");
		Button exitButton = new Button("Finalizar");
		Button showCaptureButton = new Button("Mostrar ltimas Capturas");
		Button processarTodasEntradasButton = new Button("Processar Sequncia\nEnquanto Houver Entradas");
		processarTodasEntradasButton.setTextAlignment(TextAlignment.CENTER);
		showConfigsButton = new ToggleButton("Mostrar Configuraes");
		showTextArea = new ToggleButton("Mostrar Entradas");
		Scene scene = new Scene(ROOT);
		scene.setFill(Color.TRANSPARENT);
		scene.getStylesheets().add(getClass().getResource("application.css").toExternalForm());
		processarUmaVezButton.setDefaultButton(true);
		
		caixaDeBotoes = new VBox(5d);
		caixaDeBotoes.getChildren().addAll(processarTodasEntradasButton, processarUmaVezButton, showTextArea, showConfigsButton, showCaptureButton, exitButton);
		caixaDeBotoes.setLayoutX(Carregador.posXdaCaixaDeBotoes);
		caixaDeBotoes.setLayoutY(Carregador.posYdaCaixaDeBotoes);
		
		caixaDeBotoes.setAlignment(Carregador.alinhamentoDosBotoes);
		
		textArea = new TextArea();
		ScrollPane scrollPaiDaTextArea = new ScrollPane(textArea);
		scrollPaiDaTextArea.setFitToWidth(true);
		scrollPaiDaTextArea.setFitToHeight(true);
		scrollPaiDaTextArea.setMaxWidth(Carregador.larguraDaJanelaDeEntradas);
		scrollPaiDaTextArea.setPrefWidth(Carregador.larguraDaJanelaDeEntradas);
		scrollPaiDaTextArea.setMaxHeight(Carregador.alturaDaJanelaDeEntradas);
		scrollPaiDaTextArea.setPrefHeight(Carregador.alturaDaJanelaDeEntradas);
		AnchorPane paiDoScrollDaTextArea = new AnchorPane(scrollPaiDaTextArea);
		AnchorPane.setTopAnchor(scrollPaiDaTextArea, (double) Carregador.posXdaJanelaDeEntradas);
		AnchorPane.setLeftAnchor(scrollPaiDaTextArea, (double) Carregador.posYdaJanelaDeEntradas);
		
		primaryStage.setScene(scene);
		primaryStage.setAlwaysOnTop(true);
		
		
		
		ObjectProperty<Button> botaoDefault = new SimpleObjectProperty<>(processarUmaVezButton);
		
		ObjectProperty<Button> botaoEmExecucao = new SimpleObjectProperty<>(null);
		
		final BooleanProperty processarTudo = new SimpleBooleanProperty(false);
		BooleanBinding estaEmExecucao = Carregador.sequenciaDeAcoes.ro_emExecucao.or(processarTudo);
		
		BooleanProperty aindaHaEntradasNaFila = new SimpleBooleanProperty(false);
		
		processarUmaVezButton.disableProperty().bind(botaoEmExecucao.isNotNull().and(botaoEmExecucao.isNotEqualTo(processarUmaVezButton)).and(estaEmExecucao));
		
		
		processarTodasEntradasButton.disableProperty().bind(
				botaoEmExecucao.isNotNull().and(botaoEmExecucao.isNotEqualTo(processarTodasEntradasButton)).and(estaEmExecucao)
				.or(aindaHaEntradasNaFila.not().and(estaEmExecucao.not()))
				);
		
		textArea.textProperty().addListener((obs, old, nv) -> {
			//textArea.setText(removerLinhasVazias(nv, false));
			if (nv.isEmpty() || nv.equals("\n")) {
				aindaHaEntradasNaFila.set(false);
			} else {
				aindaHaEntradasNaFila.set(true);
			}
		});
		
		
		Carregador.sequenciaDeAcoes.ro_emExecucao.addListener((obs, old, nv) -> {
			if (!nv) {
				Platform.runLater(() -> {
					if (!(textArea.getText().isEmpty() || textArea.getText().equals("\n"))) {
						if (processarTudo.get())
							executarSequencia(processarTudo);
					} else {
						processarTudo.set(false);
					}
				});
			}
		});
		processarTodasEntradasButton.setOnAction(e -> {
			processarUmaVezButton.setDefaultButton(false);
			processarTodasEntradasButton.setDefaultButton(true);
			botaoDefault.set(processarTodasEntradasButton);
				if (estaEmExecucao.get()) {
					processarTudo.set(false);
					cancelarSequencia();
				} else {
					if (!(textArea.getText().isEmpty() || textArea.getText().equals("\n"))) {
						processarTudo.set(true);
						botaoEmExecucao.set(processarTodasEntradasButton);
						executarSequencia(processarTudo);
					}
				}
		});
		
		Carregador.sequenciaDeAcoes.getGUI().addListener(new ListChangeListener<Node>() {
			@Override
			public void onChanged(Change<? extends Node> c) {
				Platform.runLater(() -> {
					ROOT.getChildren().clear();
					ROOT.getChildren().addAll(Carregador.sequenciaDeAcoes.getGUI());
					addButtons(caixaDeBotoes);
				});
			}

		});
		
		
		processarUmaVezButton.setOnAction(e -> {
			processarUmaVezButton.setDefaultButton(true);
			processarTodasEntradasButton.setDefaultButton(false);
			botaoDefault.set(processarUmaVezButton);
			botaoEmExecucao.set(processarUmaVezButton);
			executarSequencia(processarTudo);
		});
		
		Retangulo areaDaSelecao = new Retangulo(Carregador.buscadorDaCheckboxDaLinhaSelecionada.areaDaSelecao);
		Retangulo areaDaCheckbox = new Retangulo(
				new Rectangle(
						Carregador.buscadorDaCheckboxDaLinhaSelecionada.bordaEsquerdaDaAreaDeCheckbox, 
						0, 
						Carregador.buscadorDaCheckboxDaLinhaSelecionada.bordaDireitaDaAreaDeCheckbox - Carregador.buscadorDaCheckboxDaLinhaSelecionada.bordaEsquerdaDaAreaDeCheckbox, 
						Toolkit.getDefaultToolkit().getScreenSize().height));
		//fazerRedimensionavel(areaDaCheckbox);
		//fazerRedimensionavel(areaDaSelecao);
		showConfigsButton.selectedProperty().addListener((obs, old, nv) -> {
			ROOT.getChildren().clear();
			if (nv) {
				ROOT.getChildren().add(areaDaSelecao);
				ROOT.getChildren().add(areaDaCheckbox);
			} else {
				ROOT.getChildren().remove(areaDaSelecao);
				ROOT.getChildren().remove(areaDaCheckbox);
			}
			addButtons(caixaDeBotoes);
		});
		showTextArea.selectedProperty().addListener((obs, old, nv) -> {
			if (nv) ROOT.getChildren().add(paiDoScrollDaTextArea); else ROOT.getChildren().remove(paiDoScrollDaTextArea);
		});
		
		
		ROOT.getChildren().addListener(new ListChangeListener<Node>() {
			@Override
			public void onChanged(Change<? extends Node> c) {
				if (ROOT.getChildren().contains(paiDoScrollDaTextArea)) {
					scrollPaiDaTextArea.requestFocus();
					textArea.requestFocus();
				}
			}
		});
		
		Carregador.sequenciaDeAcoes.ro_emExecucao.addListener((obs, old, nv) -> {
			Platform.runLater(() -> {
				if (nv && botaoEmExecucao.isEqualTo(processarUmaVezButton).get()) processarUmaVezButton.setText("Cancelar Execuo!"); else processarUmaVezButton.setText("Processar Sequncia uma vez");
				if (nv && botaoEmExecucao.isEqualTo(processarTodasEntradasButton).get()) processarTodasEntradasButton.setText("Cancelar Execuo!"); else processarTodasEntradasButton.setText("Processar Sequncia\nEnquanto Houver Entradas");
				if (!processarTudo.get()) {
					primaryStage.requestFocus();
					botaoDefault.get().requestFocus();
				}
			});
		});
		
		addButtons(caixaDeBotoes);
		
		itensCapturados = new Group();
		ScrollPane scrollPane = new ScrollPane(itensCapturados);
		scrollPane.setStyle("-fx-background: #555555;");
		scrollPane.setFitToWidth(true);
		scrollPane.setFitToHeight(true);
		scrollPane.setPannable(true);
		scrollPane.setVbarPolicy(ScrollPane.ScrollBarPolicy.ALWAYS);
		scrollPane.setHbarPolicy(ScrollPane.ScrollBarPolicy.ALWAYS);
		
		TelaDaCaptura = new Stage();
		TelaDaCaptura.setTitle("Itens Capturados na ltima execuo do Comando de Clicar em Checkbox");
		TelaDaCaptura.setScene(new Scene(scrollPane, Carregador.larguraDaJanelaDeCapturas, Carregador.alturaDaJanelaDeCapturas));
		showCaptureButton.setOnAction(e -> {
			mostrarTelaDaCaptura();
		});
		
		exitButton.setOnAction(e -> System.exit(0));
		primaryStage.setOnHidden(e -> System.exit(0));
		
		primaryStage.show();
	}

	public void fazerRedimensionavel(Retangulo retangulo) { //TODO continuar implementando e corrigindo
		retangulo.setCursor(Cursor.H_RESIZE);
		retangulo.setOnMouseEntered(e -> retangulo.acender());
		retangulo.setOnMouseExited(e -> retangulo.apagar());
		retangulo.setOnMouseDragged(e -> {
			double distanciaBordaEsquerda = e.getX() - retangulo.getX();
			double distanciaBordaDireita = e.getX() - (retangulo.getX()+retangulo.getWidth());
			if (distanciaBordaEsquerda < 0) distanciaBordaEsquerda = distanciaBordaEsquerda*-1;
			if (distanciaBordaDireita < 0) distanciaBordaDireita = distanciaBordaDireita*-1;
			if (distanciaBordaEsquerda <= 5) {
				if (distanciaBordaDireita >=6) {
					double posBordaDireita = retangulo.getX()+retangulo.getWidth();
					retangulo.setWidth(posBordaDireita - e.getX());
					retangulo.setX(e.getX());
				}
				System.out.println("Est na Borda Esquerda!");
			} else if (distanciaBordaDireita <= 5) {
				if (distanciaBordaEsquerda >=6) {
					retangulo.setWidth(e.getX() - retangulo.getX());
				}
				System.out.println("Est na Borda Direita!");
			}
		});
	}


	public static String removerLinhasVazias(String string, boolean removerLinhaVaziaNoFilalSeHouver) {
		StringBuilder novaString = new StringBuilder(); 
		String[] split = string.split("\n");
		for (String linha : split) {
			if (!linha.isEmpty()) novaString.append(linha+"\n");
		}
		String finalString = novaString.toString();
		if (finalString.endsWith("\n") && removerLinhaVaziaNoFilalSeHouver) 
			finalString = finalString.substring(0, finalString.length() -1);
		if (!removerLinhaVaziaNoFilalSeHouver && !string.isEmpty() && !string.endsWith("\n"))
			finalString = finalString.substring(0, finalString.length() -1);
		return finalString;
	}

	/**Executa a sequncia se no estiver em execuo, ou cancela ela se j estiver. */
	public void executarSequencia(BooleanProperty processarTudo) {
		showConfigsButton.setSelected(false);
		showTextArea.setSelected(false);
		try {
			if (Carregador.sequenciaDeAcoes.ro_emExecucao.get()) {
				processarTudo.set(false);
				cancelarSequencia();
			} else {
				Carregador.sequenciaDeAcoes.executar(Carregador.robo); 
				ROOT.getChildren().clear();
				addButtons(caixaDeBotoes);
			}
		} catch (Exception ex) {
			Carregador.saidaComErro(ex);
		}
	}

	public void cancelarSequencia() {
		Carregador.sequenciaDeAcoes.cancelar();
		ROOT.getChildren().clear();
		addButtons(caixaDeBotoes);
		System.out.println("Cancelado!");
	}

	protected void addButtons(VBox caixaDeBotoes) {
		ROOT.getChildren().add(caixaDeBotoes);
		
	}

	public static void mostrarTelaDaCaptura() {
		Platform.runLater(() -> {
			atualizarItensCapturados(itensCapturados);
			TelaDaCaptura.show();
		});
	}

	private static void atualizarItensCapturados(Group itensCapturados) {
		itensCapturados.getChildren().clear();
		double incY = 0;
		if (Carregador.buscadorDaCheckboxDaLinhaSelecionada.selecaoEncontrada.get() != null) {
			Image image = SwingFXUtils.toFXImage(Carregador.buscadorDaCheckboxDaLinhaSelecionada.selecaoEncontrada.get(), null);
			ImageView imageView = new ImageView(image);
			imageView.setTranslateX(20);
			imageView.setTranslateY(20);
			itensCapturados.getChildren().add(imageView);
			incY = image.getHeight();
		}
		
		if (Carregador.buscadorDaCheckboxDaLinhaSelecionada.areaDaSelecaoBuscada.get() != null) {
			Image image = SwingFXUtils.toFXImage(Carregador.buscadorDaCheckboxDaLinhaSelecionada.areaDaSelecaoBuscada.get(), null);
			ImageView imageView = new ImageView(image);
			imageView.setTranslateX(20);
			imageView.setTranslateY(20+incY+10);
			itensCapturados.getChildren().add(imageView);

			if (Carregador.buscadorDaCheckboxDaLinhaSelecionada.areaComCheckboxBuscada.get() != null) {
				Image image2 = SwingFXUtils.toFXImage(Carregador.buscadorDaCheckboxDaLinhaSelecionada.areaComCheckboxBuscada.get(), null);
				ImageView imageView2 = new ImageView(image2);
				imageView2.setTranslateX(image.getWidth() + 40);
				imageView2.setTranslateY(20);
				itensCapturados.getChildren().add(imageView2);

				Image image3 = SwingFXUtils.toFXImage(Carregador.checkBoxImage, null);
				ImageView imageView3 = new ImageView(image3);
				imageView3.setTranslateX(image.getWidth() + 40);
				imageView3.setTranslateY(image2.getHeight() + 40);
				itensCapturados.getChildren().add(imageView3);
			}
		}
	}
	
	public static void mostrarMensagem(AlertType alertType, String title, String message) {
		Platform.runLater(() -> {
			Alert alert = new Alert(alertType);
			alert.setTitle(title);
			alert.setHeaderText(null);
			alert.setContentText(message);
			alert.showAndWait();
		});
	}

	public static void main(String[] args) {
		launch(args);
	}

	/**Avana para Prxima-Entrada tornando-a a Entrada-Atual, deleta a Ex-Entrada-Atual.*/
	public static void proximaEntrada() {
	    entradaAtual = proximaEntrada(textArea.textProperty());
	    System.out.println("Processar Entrada: "+entradaAtual);
	}
	
	static String proximaEntrada(StringProperty entradas) {
		while (entradas.get().startsWith("\n")) entradas.set(entradas.get().substring(1));
		if (entradas.get().isEmpty()) {
			return "";
		}
		String textOriginal = entradas.get();
		String[] split = textOriginal.split("\n");
		if (split.length == 0) {
			entradas.set("");
			return "";
		}
		String proxEntrada = split[0];
		entradas.set(textOriginal.substring(proxEntrada.length()));
		while (entradas.get().startsWith("\n")) entradas.set(entradas.get().substring(1));
	    return proxEntrada;
	}
}
