package main;

import static org.junit.jupiter.api.Assertions.assertEquals;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;

import javafx.beans.property.SimpleStringProperty;

class TestMainFX {
	
	@Nested
	@DisplayName("testar 'proximaEntrada(StringProperty)'")
	class MetodoPromixaEntrada {
		
		@Test
		@DisplayName("ao receber vazio")
		final void test1() {
			SimpleStringProperty textAreaDeEntradasVazia = new SimpleStringProperty("");
			
			String novaEntradaAtual = MainFX.proximaEntrada(textAreaDeEntradasVazia);
			
			assertEquals("", novaEntradaAtual);
			assertEquals("", textAreaDeEntradasVazia.get());
		}
		
		@Test
		@DisplayName("ao receber \\n")
		final void test2() {
			SimpleStringProperty textAreaDeEntradasVazia = new SimpleStringProperty("\n");
			
			String novaEntradaAtual = MainFX.proximaEntrada(textAreaDeEntradasVazia);
			
			assertEquals("", novaEntradaAtual);
			assertEquals("", textAreaDeEntradasVazia.get());
		}
		
		@Test
		@DisplayName("ao receber \\n\\n")
		final void test3() {
			SimpleStringProperty textAreaDeEntradasVazia = new SimpleStringProperty("\n\n");
			
			String novaEntradaAtual = MainFX.proximaEntrada(textAreaDeEntradasVazia);
			
			assertEquals("", novaEntradaAtual);
			assertEquals("", textAreaDeEntradasVazia.get());
		}
		
		@Test
		@DisplayName("ao receber \\n\\n\\n")
		final void test4() {
			SimpleStringProperty textAreaDeEntradasVazia = new SimpleStringProperty("\n\n\n");
			
			String novaEntradaAtual = MainFX.proximaEntrada(textAreaDeEntradasVazia);
			
			assertEquals("", novaEntradaAtual);
			assertEquals("", textAreaDeEntradasVazia.get());
		}
		
		@Test
		@DisplayName("ao receber \\nEntrada1\\n\\n")
		final void test5() {
			SimpleStringProperty textAreaDeEntradasVazia = new SimpleStringProperty("\nEntrada1\n\n");
			
			String novaEntradaAtual = MainFX.proximaEntrada(textAreaDeEntradasVazia);
			
			assertEquals("Entrada1", novaEntradaAtual);
			assertEquals("", textAreaDeEntradasVazia.get());
		}
		
		@Test
		@DisplayName("ao receber \\nEntrada1\\nEntrada2\\n\\n")
		final void test6() {
			SimpleStringProperty textAreaDeEntradasVazia = new SimpleStringProperty("\nEntrada1\nEntrada2\n\n");
			
			String novaEntradaAtual = MainFX.proximaEntrada(textAreaDeEntradasVazia);
			
			assertEquals("Entrada1", novaEntradaAtual);
			assertEquals("Entrada2\n\n", textAreaDeEntradasVazia.get());
		}
		
		@Test
		@DisplayName("ao receber \\nEntrada2\\n\\n")
		final void test10() {
			SimpleStringProperty textAreaDeEntradasVazia = new SimpleStringProperty("\nEntrada2\n\n");
			
			String novaEntradaAtual = MainFX.proximaEntrada(textAreaDeEntradasVazia);
			
			assertEquals("Entrada2", novaEntradaAtual);
			assertEquals("", textAreaDeEntradasVazia.get());
		}
		
		@Test
		@DisplayName("ao receber \\n\\nEntrada2\\n\\n")
		final void test11() {
			SimpleStringProperty textAreaDeEntradasVazia = new SimpleStringProperty("\n\nEntrada2\n\n");
			
			String novaEntradaAtual = MainFX.proximaEntrada(textAreaDeEntradasVazia);
			
			assertEquals("Entrada2", novaEntradaAtual);
			assertEquals("", textAreaDeEntradasVazia.get());
		}
		
		@Test
		@DisplayName("ao receber \\n\\n\\nEntrada2\\n\\n")
		final void test12() {
			SimpleStringProperty textAreaDeEntradasVazia = new SimpleStringProperty("\n\n\nEntrada2\n\n");
			
			String novaEntradaAtual = MainFX.proximaEntrada(textAreaDeEntradasVazia);
			
			assertEquals("Entrada2", novaEntradaAtual);
			assertEquals("", textAreaDeEntradasVazia.get());
		}
		
		@Test
		@DisplayName("ao receber Entrada0\\nEntrada1\\nEntrada2\\n\\n")
		final void test7() {
			SimpleStringProperty textAreaDeEntradasVazia = new SimpleStringProperty("Entrada0\nEntrada1\nEntrada2\n\n");
			
			String novaEntradaAtual = MainFX.proximaEntrada(textAreaDeEntradasVazia);
			
			assertEquals("Entrada0", novaEntradaAtual);
			assertEquals("Entrada1\nEntrada2\n\n", textAreaDeEntradasVazia.get());
		}
		
		@Test
		@DisplayName("ao receber Entrada0")
		final void test8() {
			SimpleStringProperty textAreaDeEntradasVazia = new SimpleStringProperty("Entrada0");
			
			String novaEntradaAtual = MainFX.proximaEntrada(textAreaDeEntradasVazia);
			
			assertEquals("Entrada0", novaEntradaAtual);
			assertEquals("", textAreaDeEntradasVazia.get());
		}
		
		@Test
		@DisplayName("ao receber Entrada0\n")
		final void test9() {
			SimpleStringProperty textAreaDeEntradasVazia = new SimpleStringProperty("Entrada0\n");
			
			String novaEntradaAtual = MainFX.proximaEntrada(textAreaDeEntradasVazia);
			
			assertEquals("Entrada0", novaEntradaAtual);
			assertEquals("", textAreaDeEntradasVazia.get());
		}
	}
	
	@Test
	final void testRemoverLinhasVazias() {
		assertEquals("", MainFX.removerLinhasVazias("", true));
		assertEquals("aaa", MainFX.removerLinhasVazias("aaa", true));
		assertEquals("aaa", MainFX.removerLinhasVazias("aaa\n", true));
		assertEquals("aaa", MainFX.removerLinhasVazias("aaa\n\n", true));
		assertEquals("aaa", MainFX.removerLinhasVazias("aaa\n\n\n", true));
		assertEquals("aaa\nbbb", MainFX.removerLinhasVazias("aaa\nbbb", true));
		assertEquals("aaa\nbbb", MainFX.removerLinhasVazias("aaa\nbbb\n", true));
		assertEquals("bbb", MainFX.removerLinhasVazias("\nbbb\n", true));
		assertEquals("bbb\nccc", MainFX.removerLinhasVazias("\nbbb\nccc", true));
		assertEquals("bbb\nccc\nddd", MainFX.removerLinhasVazias("\nbbb\nccc\n\nddd", true));
		
		assertEquals("", MainFX.removerLinhasVazias("", false));
		assertEquals("aaa", MainFX.removerLinhasVazias("aaa", false));
		assertEquals("aaa\n", MainFX.removerLinhasVazias("aaa\n", false));
		assertEquals("aaa\n", MainFX.removerLinhasVazias("aaa\n\n", false));
		assertEquals("aaa\n", MainFX.removerLinhasVazias("aaa\n\n\n", false));
		assertEquals("aaa\nbbb", MainFX.removerLinhasVazias("aaa\nbbb", false));
		assertEquals("aaa\nbbb\n", MainFX.removerLinhasVazias("aaa\nbbb\n", false));
		assertEquals("bbb\n", MainFX.removerLinhasVazias("\nbbb\n", false));
		assertEquals("bbb\nccc", MainFX.removerLinhasVazias("\nbbb\nccc", false));
		assertEquals("bbb\nccc\nddd", MainFX.removerLinhasVazias("\nbbb\nccc\n\nddd", false));
	}
	
}
